package sun.swing;

import javax.swing.UIManager;
import javax.swing.JRadioButton;
import javax.swing.JCheckBox;
import javax.swing.Icon;
import java.awt.Insets;
import javax.swing.plaf.UIResource;
import javax.swing.AbstractButton;
import javax.swing.JLabel;
import java.awt.Container;
import javax.swing.JComponent;
import javax.swing.LayoutStyle;

public class DefaultLayoutStyle extends LayoutStyle
{
    private static final DefaultLayoutStyle INSTANCE;
    
    public DefaultLayoutStyle() {
    }
    
    public static LayoutStyle getInstance() {
        return DefaultLayoutStyle.INSTANCE;
    }
    
    @Override
    public int getPreferredGap(JComponent component, JComponent component2, LayoutStyle.ComponentPlacement componentPlacement, int n, Container container) {
        if (component == null || component2 == null || componentPlacement == null) {
            throw new NullPointerException();
        }
        this.checkPosition(n);
        if (componentPlacement == LayoutStyle.ComponentPlacement.INDENT && (n == 3 || n == 7)) {
            int indent = this.getIndent(component, n);
            if (indent > 0) {
                return indent;
            }
        }
        return (componentPlacement == LayoutStyle.ComponentPlacement.UNRELATED) ? 12 : 6;
    }
    
    @Override
    public int getContainerGap(JComponent component, int n, Container container) {
        if (component == null) {
            throw new NullPointerException();
        }
        this.checkPosition(n);
        return 6;
    }
    
    protected boolean isLabelAndNonlabel(JComponent component, JComponent component2, int n) {
        if (n == 3 || n == 7) {
            int n2 = component instanceof JLabel;
            int n3 = component2 instanceof JLabel;
            return (n2 || n3) && n2 != n3;
        }
        return false;
    }
    
    protected int getButtonGap(JComponent component, JComponent component2, int n, int n2) {
        n2 -= this.getButtonGap(component, n);
        if (n2 > 0) {
            n2 -= this.getButtonGap(component2, this.flipDirection(n));
        }
        if (n2 < 0) {
            return 0;
        }
        return n2;
    }
    
    protected int getButtonGap(JComponent component, int n, int a) {
        a -= this.getButtonGap(component, n);
        return Math.max(a, 0);
    }
    
    public int getButtonGap(JComponent component, int n) {
        String uiClassID = component.getUIClassID();
        if ((uiClassID == "CheckBoxUI" || uiClassID == "RadioButtonUI") && !((AbstractButton)component).isBorderPainted() && component.getBorder() instanceof UIResource) {
            return this.getInset(component, n);
        }
        return 0;
    }
    
    private void checkPosition(int n) {
        if (n != 1 && n != 5 && n != 7 && n != 3) {
            throw new IllegalArgumentException();
        }
        return;
    }
    
    protected int flipDirection(int n) {
        switch (n) {
            case 1: {
                return 5;
            }
            case 5: {
                return 1;
            }
            case 3: {
                return 7;
            }
            case 7: {
                return 3;
            }
            default: {
                assert false;
                return 0;
            }
        }
    }
    
    protected int getIndent(JComponent component, int n) {
        String uiClassID = component.getUIClassID();
        if (uiClassID == "CheckBoxUI" || uiClassID == "RadioButtonUI") {
            AbstractButton abstractButton = (AbstractButton)component;
            Insets insets = component.getInsets();
            Icon icon = this.getIcon(abstractButton);
            int iconTextGap = abstractButton.getIconTextGap();
            if (this.isLeftAligned(abstractButton, n)) {
                return insets.left + icon.getIconWidth() + iconTextGap;
            }
            if (this.isRightAligned(abstractButton, n)) {
                return insets.right + icon.getIconWidth() + iconTextGap;
            }
        }
        return 0;
    }
    
    private Icon getIcon(AbstractButton abstractButton) {
        Icon icon = abstractButton.getIcon();
        if (icon != null) {
            return icon;
        }
        String key = null;
        if ((abstractButton instanceof JCheckBox)) {
            key = "CheckBox.icon";
        }
        else if ((abstractButton instanceof JRadioButton)) {
            key = "RadioButton.icon";
        }
        if (key != null) {
            Object value = UIManager.get(key);
            if ((value instanceof Icon)) {
                return (Icon)value;
            }
        }
        return null;
    }
    
    private boolean isLeftAligned(AbstractButton abstractButton, int n) {
        if (n == 7) {
            int leftToRight = abstractButton.getComponentOrientation().isLeftToRight();
            int horizontalAlignment = abstractButton.getHorizontalAlignment();
            return (leftToRight && (horizontalAlignment == 2 || horizontalAlignment == 10)) || (leftToRight == 0 && horizontalAlignment == 11);
        }
        return false;
    }
    
    private boolean isRightAligned(AbstractButton abstractButton, int n) {
        if (n == 3) {
            int leftToRight = abstractButton.getComponentOrientation().isLeftToRight();
            int horizontalAlignment = abstractButton.getHorizontalAlignment();
            return (leftToRight && (horizontalAlignment == 4 || horizontalAlignment == 11)) || (leftToRight == 0 && horizontalAlignment == 10);
        }
        return false;
    }
    
    private int getInset(JComponent component, int n) {
        return this.getInset(component.getInsets(), n);
    }
    
    private int getInset(Insets insets, int n) {
        if (insets == null) {
            return 0;
        }
        switch (n) {
            case 1: {
                return insets.top;
            }
            case 5: {
                return insets.bottom;
            }
            case 3: {
                return insets.right;
            }
            case 7: {
                return insets.left;
            }
            default: {
                assert false;
                return 0;
            }
        }
    }
    
    static {
        INSTANCE = new DefaultLayoutStyle();
    }
}
